@@ -159,6 +159,8 @@ struct _MetaDisplay
   Atom atom_net_wm_action_maximize_vert;
   Atom atom_net_wm_action_change_desktop;
   Atom atom_net_wm_action_close;
+  Atom atom_net_wm_action_above;
+  Atom atom_net_wm_action_below;
   Atom atom_net_wm_state_above;
   Atom atom_net_wm_state_below;
   Atom atom_net_startup_id;
@@ -414,6 +416,10 @@ void          meta_display_unmanage_windows_for_screen (MetaDisplay *display,
                                                         MetaScreen  *screen,
                                                         guint32      timestamp);
 
+/* Utility function to compare the stacking of two windows */
+int           meta_display_stack_cmp           (const void *a,
+                                                const void *b);
+
 /* A given MetaWindow may have various X windows that "belong"
  * to it, such as the frame window.
  */
